@@ -3,7 +3,7 @@ ldapobject.py - wraps class _ldap.LDAPObject
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: ldapobject.py,v 1.109 2010/06/03 12:26:39 stroeder Exp $
+\$Id: ldapobject.py,v 1.120 2011/04/18 13:51:25 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -25,7 +25,6 @@ __all__ = [
   'SimpleLDAPObject',
   'NonblockingLDAPObject',
   'ReconnectLDAPObject',
-  'SmartLDAPObject'
 ]
 
 
@@ -36,7 +35,9 @@ if __debug__:
 import sys,time,_ldap,ldap,ldap.functions
 
 from ldap.schema import SCHEMA_ATTRS
-from ldap.controls import LDAPControl,DecodeControlTuples,EncodeControlTuples
+from ldap.controls import LDAPControl,DecodeControlTuples,RequestControlTuples
+from ldap.extop import ExtendedRequest,ExtendedResponse
+
 from ldap import LDAPError
 
 
@@ -131,15 +132,15 @@ class SimpleLDAPObject:
 
   def abandon_ext(self,msgid,serverctrls=None,clientctrls=None):
     """
-    abandon_ext(msgid[,serverctrls=None[,clientctrls=None]]) -> None    
-    abandon(msgid) -> None    
+    abandon_ext(msgid[,serverctrls=None[,clientctrls=None]]) -> None
+    abandon(msgid) -> None
         Abandons or cancels an LDAP operation in progress. The msgid should
         be the message id of an outstanding LDAP operation as returned
         by the asynchronous methods search(), modify() etc.  The caller
         can expect that the result of an abandoned operation will not be
         returned from a future call to result().
     """
-    return self._ldap_call(self._l.abandon_ext,msgid,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.abandon_ext,msgid,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def abandon(self,msgid):
     return self.abandon_ext(msgid,None,None)
@@ -148,14 +149,14 @@ class SimpleLDAPObject:
     """
     cancel(cancelid[,serverctrls=None[,clientctrls=None]]) -> int
         Send cancels extended operation for an LDAP operation specified by cancelid.
-	The cancelid should be the message id of an outstanding LDAP operation as returned
+        The cancelid should be the message id of an outstanding LDAP operation as returned
         by the asynchronous methods search(), modify() etc.  The caller
         can expect that the result of an abandoned operation will not be
         returned from a future call to result().
-	In opposite to abandon() this extended operation gets an result from
-	the server and thus should be preferred if the server supports it.
+        In opposite to abandon() this extended operation gets an result from
+        the server and thus should be preferred if the server supports it.
     """
-    return self._ldap_call(self._l.cancel,cancelid,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.cancel,cancelid,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def cancel_s(self,cancelid,serverctrls=None,clientctrls=None):
     msgid = self.cancel(cancelid,serverctrls,clientctrls)
@@ -173,11 +174,12 @@ class SimpleLDAPObject:
         The parameter modlist is similar to the one passed to modify(),
         except that no operation integer need be included in the tuples.
     """
-    return self._ldap_call(self._l.add_ext,dn,modlist,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.add_ext,dn,modlist,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def add_ext_s(self,dn,modlist,serverctrls=None,clientctrls=None):
     msgid = self.add_ext(dn,modlist,serverctrls,clientctrls)
-    return self.result(msgid,all=1,timeout=self.timeout)
+    resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
+    return resp_type, resp_data, resp_msgid, resp_ctrls
 
   def add(self,dn,modlist):
     """
@@ -197,14 +199,15 @@ class SimpleLDAPObject:
     """
     simple_bind([who='' [,cred='']]) -> int
     """
-    return self._ldap_call(self._l.simple_bind,who,cred,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.simple_bind,who,cred,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def simple_bind_s(self,who='',cred='',serverctrls=None,clientctrls=None):
     """
     simple_bind_s([who='' [,cred='']]) -> None
     """
     msgid = self.simple_bind(who,cred,serverctrls,clientctrls)
-    return self.result(msgid,all=1,timeout=self.timeout)
+    resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
+    return resp_type, resp_data, resp_msgid, resp_ctrls
 
   def bind(self,who,cred,method=ldap.AUTH_SIMPLE):
     """
@@ -224,14 +227,14 @@ class SimpleLDAPObject:
     """
     sasl_interactive_bind_s(who, auth) -> None
     """
-    return self._ldap_call(self._l.sasl_interactive_bind_s,who,auth,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls),sasl_flags)
+    return self._ldap_call(self._l.sasl_interactive_bind_s,who,auth,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls),sasl_flags)
 
   def compare_ext(self,dn,attr,value,serverctrls=None,clientctrls=None):
     """
     compare_ext(dn, attr, value [,serverctrls=None[,clientctrls=None]]) -> int
-    compare_ext_s(dn, attr, value [,serverctrls=None[,clientctrls=None]]) -> int    
+    compare_ext_s(dn, attr, value [,serverctrls=None[,clientctrls=None]]) -> int
     compare(dn, attr, value) -> int
-    compare_s(dn, attr, value) -> int    
+    compare_s(dn, attr, value) -> int
         Perform an LDAP comparison between the attribute named attr of
         entry dn, and the value value. The synchronous form returns 0
         for false, or 1 for true.  The asynchronous form returns the
@@ -244,12 +247,12 @@ class SimpleLDAPObject:
         A design bug in the library prevents value from containing
         nul characters.
     """
-    return self._ldap_call(self._l.compare_ext,dn,attr,value,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.compare_ext,dn,attr,value,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def compare_ext_s(self,dn,attr,value,serverctrls=None,clientctrls=None):
     msgid = self.compare_ext(dn,attr,value,serverctrls,clientctrls)
     try:
-      self.result(msgid,all=1,timeout=self.timeout)
+      resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
     except ldap.COMPARE_TRUE:
       return 1
     except ldap.COMPARE_FALSE:
@@ -272,11 +275,12 @@ class SimpleLDAPObject:
         form returns the message id of the initiated request, and the
         result can be obtained from a subsequent call to result().
     """
-    return self._ldap_call(self._l.delete_ext,dn,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.delete_ext,dn,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def delete_ext_s(self,dn,serverctrls=None,clientctrls=None):
     msgid = self.delete_ext(dn,serverctrls,clientctrls)
-    return self.result(msgid,all=1,timeout=self.timeout)
+    resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
+    return resp_type, resp_data, resp_msgid, resp_ctrls
 
   def delete(self,dn):
     return self.delete_ext(dn,None,None)
@@ -284,33 +288,65 @@ class SimpleLDAPObject:
   def delete_s(self,dn):
     return self.delete_ext_s(dn,None,None)
 
+  def extop(self,extreq,serverctrls=None,clientctrls=None):
+    """
+    extop(extreq[,serverctrls=None[,clientctrls=None]]]) -> int
+    extop_s(extreq[,serverctrls=None[,clientctrls=None[,extop_resp_class=None]]]]) ->
+        (respoid,respvalue)
+        Performs an LDAP extended operation. The asynchronous
+        form returns the message id of the initiated request, and the
+        result can be obtained from a subsequent call to result3().
+        The extreq is an instance of class ldap.extop.ExtendedRequest.
+
+        If argument extop_resp_class is set to a sub-class of
+        ldap.extop.ExtendedResponse this class is used to return an
+        object of this class instead of a raw BER value in respvalue.
+    """
+    return self._ldap_call(self._l.extop,extreq.requestName,extreq.encodedRequestValue(),RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
+
+  def extop_result(self,msgid=ldap.RES_ANY,all=1,timeout=None):
+    resulttype,msg,msgid,respctrls,respoid,respvalue = self.result4(msgid,all=1,timeout=self.timeout,add_ctrls=1,add_intermediates=1,add_extop=1)
+    return (respoid,respvalue)
+
+  def extop_s(self,extreq,serverctrls=None,clientctrls=None,extop_resp_class=None):
+    msgid = self.extop(extreq,serverctrls,clientctrls)
+    res = self.extop_result(msgid,all=1,timeout=self.timeout)
+    if extop_resp_class:
+      respoid,respvalue = res
+      if extop_resp_class.responseName!=respoid:
+        raise ldap.PROTOCOL_ERROR("Wrong OID in extended response! Expected %s, got %s" % (extop_resp_class.responseName,respoid))
+      return extop_resp_class(extop_resp_class.responseName,respvalue)
+    else:
+      return res
+
   def modify_ext(self,dn,modlist,serverctrls=None,clientctrls=None):
     """
     modify_ext(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> int
     """
-    return self._ldap_call(self._l.modify_ext,dn,modlist,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.modify_ext,dn,modlist,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def modify_ext_s(self,dn,modlist,serverctrls=None,clientctrls=None):
     msgid = self.modify_ext(dn,modlist,serverctrls,clientctrls)
-    return self.result(msgid,all=1,timeout=self.timeout)
+    resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
+    return resp_type, resp_data, resp_msgid, resp_ctrls
 
   def modify(self,dn,modlist):
     """
     modify(dn, modlist) -> int
-    modify_s(dn, modlist) -> None    
+    modify_s(dn, modlist) -> None
     modify_ext(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> int
     modify_ext_s(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> None
         Performs an LDAP modify operation on an entry's attributes.
         dn is the DN of the entry to modify, and modlist is the list
         of modifications to make to the entry.
 
-	Each element of the list modlist should be a tuple of the form
-	(mod_op,mod_type,mod_vals), where mod_op is the operation (one of
-	MOD_ADD, MOD_DELETE, MOD_INCREMENT or MOD_REPLACE), mod_type is a
-	string indicating the attribute type name, and mod_vals is either a
-	string value or a list of string values to add, delete, increment by or
-	replace respectively.  For the delete operation, mod_vals may be None
-	indicating that all attributes are to be deleted.
+        Each element of the list modlist should be a tuple of the form
+        (mod_op,mod_type,mod_vals), where mod_op is the operation (one of
+        MOD_ADD, MOD_DELETE, MOD_INCREMENT or MOD_REPLACE), mod_type is a
+        string indicating the attribute type name, and mod_vals is either a
+        string value or a list of string values to add, delete, increment by or
+        replace respectively.  For the delete operation, mod_vals may be None
+        indicating that all attributes are to be deleted.
 
         The asynchronous modify() returns the message id of the
         initiated request.
@@ -324,7 +360,7 @@ class SimpleLDAPObject:
   def modrdn(self,dn,newrdn,delold=1):
     """
     modrdn(dn, newrdn [,delold=1]) -> int
-    modrdn_s(dn, newrdn [,delold=1]) -> None    
+    modrdn_s(dn, newrdn [,delold=1]) -> None
         Perform a modify RDN operation. These routines take dn, the
         DN of the entry whose RDN is to be changed, and newrdn, the
         new RDN to give to the entry. The optional parameter delold
@@ -341,11 +377,12 @@ class SimpleLDAPObject:
     return self.rename_s(dn,newrdn,None,delold)
 
   def passwd(self,user,oldpw,newpw,serverctrls=None,clientctrls=None):
-    return self._ldap_call(self._l.passwd,user,oldpw,newpw,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.passwd,user,oldpw,newpw,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def passwd_s(self,user,oldpw,newpw,serverctrls=None,clientctrls=None):
     msgid = self.passwd(user,oldpw,newpw,serverctrls,clientctrls)
-    return self.result(msgid,all=1,timeout=self.timeout)
+    resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
+    return resp_type, resp_data, resp_msgid, resp_ctrls
 
   def rename(self,dn,newrdn,newsuperior=None,delold=1,serverctrls=None,clientctrls=None):
     """
@@ -362,11 +399,12 @@ class SimpleLDAPObject:
         This actually corresponds to the rename* routines in the
         LDAP-EXT C API library.
     """
-    return self._ldap_call(self._l.rename,dn,newrdn,newsuperior,delold,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.rename,dn,newrdn,newsuperior,delold,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def rename_s(self,dn,newrdn,newsuperior=None,delold=1,serverctrls=None,clientctrls=None):
     msgid = self.rename(dn,newrdn,newsuperior,delold,serverctrls,clientctrls)
-    return self.result(msgid,all=1,timeout=self.timeout)
+    resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
+    return resp_type, resp_data, resp_msgid, resp_ctrls
 
   def result(self,msgid=ldap.RES_ANY,all=1,timeout=None):
     """
@@ -419,24 +457,38 @@ class SimpleLDAPObject:
         If a timeout occurs, a TIMEOUT exception is raised, unless
         polling (timeout = 0), in which case (None, None) is returned.
     """
-    res_type,res_data,res_msgid = self.result2(msgid,all,timeout)
-    return res_type,res_data
+    resp_type, resp_data, resp_msgid = self.result2(msgid,all,timeout)
+    return resp_type, resp_data
 
   def result2(self,msgid=ldap.RES_ANY,all=1,timeout=None):
-    res_type, res_data, res_msgid, srv_ctrls = self.result3(msgid,all,timeout)
-    return res_type, res_data, res_msgid
- 
-  def result3(self,msgid=ldap.RES_ANY,all=1,timeout=None):
+    resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all,timeout)
+    return resp_type, resp_data, resp_msgid
+
+  def result3(self,msgid=ldap.RES_ANY,all=1,timeout=None,resp_ctrl_classes=None):
+    resp_type, resp_data, resp_msgid, decoded_resp_ctrls, retoid, retval = self.result4(
+      msgid,all,timeout,
+      add_ctrls=0,add_intermediates=0,add_extop=0,
+      resp_ctrl_classes=resp_ctrl_classes
+    )
+    return resp_type, resp_data, resp_msgid, decoded_resp_ctrls
+
+  def result4(self,msgid=ldap.RES_ANY,all=1,timeout=None,add_ctrls=0,add_intermediates=0,add_extop=0,resp_ctrl_classes=None):
     if timeout is None:
       timeout = self.timeout
-    ldap_result = self._ldap_call(self._l.result3,msgid,all,timeout)
+    ldap_result = self._ldap_call(self._l.result4,msgid,all,timeout,add_ctrls,add_intermediates,add_extop)
     if ldap_result is None:
-      rtype, rdata, rmsgid, decoded_serverctrls = (None,None,None,None)
+        resp_type, resp_data, resp_msgid, resp_ctrls, resp_name, resp_value = (None,None,None,None,None,None)
     else:
-      rtype, rdata, rmsgid, serverctrls = ldap_result
-      decoded_serverctrls = DecodeControlTuples(serverctrls)
-    return rtype, rdata, rmsgid, decoded_serverctrls
- 
+      if len(ldap_result)==4:
+        resp_type, resp_data, resp_msgid, resp_ctrls = ldap_result
+        resp_name, resp_value = None,None
+      else:
+        resp_type, resp_data, resp_msgid, resp_ctrls, resp_name, resp_value = ldap_result
+      if add_ctrls:
+        resp_data = [ (t,r,DecodeControlTuples(c,resp_ctrl_classes)) for t,r,c in resp_data ]
+    decoded_resp_ctrls = DecodeControlTuples(resp_ctrls,resp_ctrl_classes)
+    return resp_type, resp_data, resp_msgid, decoded_resp_ctrls, resp_name, resp_value
+
   def search_ext(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
     """
     search(base, scope [,filterstr='(objectClass=*)' [,attrlist=None [,attrsonly=0]]]) -> int
@@ -486,8 +538,8 @@ class SimpleLDAPObject:
       self._l.search_ext,
       base,scope,filterstr,
       attrlist,attrsonly,
-      EncodeControlTuples(serverctrls),
-      EncodeControlTuples(clientctrls),
+      RequestControlTuples(serverctrls),
+      RequestControlTuples(clientctrls),
       timeout,sizelimit,
     )
 
@@ -504,48 +556,37 @@ class SimpleLDAPObject:
   def search_st(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,timeout=-1):
     return self.search_ext_s(base,scope,filterstr,attrlist,attrsonly,None,None,timeout)
 
-  def set_cache_options(self,*args,**kwargs):
-    """
-    set_cache_options(option) -> None    
-        Changes the caching behaviour. Currently supported options are
-            CACHE_OPT_CACHENOERRS, which suppresses caching of requests
-                that resulted in an error, and
-            CACHE_OPT_CACHEALLERRS, which enables caching of all requests.
-        The default behaviour is not to cache requests that result in
-        errors, except those that result in a SIZELIMIT_EXCEEDED exception.
-    """
-    return self._ldap_call(self._l.set_cache_options,*args,**kwargs)
-
   def start_tls_s(self):
     """
-    start_tls_s() -> None    
+    start_tls_s() -> None
     Negotiate TLS with server. The `version' attribute must have been
     set to VERSION3 before calling start_tls_s.
     If TLS could not be started an exception will be raised.
     """
     return self._ldap_call(self._l.start_tls_s)
-  
+
   def unbind_ext(self,serverctrls=None,clientctrls=None):
     """
-    unbind() -> int    
+    unbind() -> int
     unbind_s() -> None
-    unbind_ext() -> int    
+    unbind_ext() -> int
     unbind_ext_s() -> None
         This call is used to unbind from the directory, terminate
         the current association, and free resources. Once called, the
         connection to the LDAP server is closed and the LDAP object
         is invalid. Further invocation of methods on the object will
         yield an exception.
-    
+
         The unbind and unbind_s methods are identical, and are
         synchronous in nature
     """
-    return self._ldap_call(self._l.unbind_ext,EncodeControlTuples(serverctrls),EncodeControlTuples(clientctrls))
+    return self._ldap_call(self._l.unbind_ext,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def unbind_ext_s(self,serverctrls=None,clientctrls=None):
     msgid = self.unbind_ext(serverctrls,clientctrls)
     if msgid!=None:
-      return self.result(msgid,all=1,timeout=self.timeout)
+      resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
+      return resp_type, resp_data, resp_msgid, resp_ctrls
 
   def unbind(self):
     return self.unbind_ext(None,None)
@@ -564,7 +605,7 @@ class SimpleLDAPObject:
 
   def set_option(self,option,invalue):
     if option==ldap.OPT_SERVER_CONTROLS or option==ldap.OPT_CLIENT_CONTROLS:
-      invalue = EncodeControlTuples(invalue)
+      invalue = RequestControlTuples(invalue)
     return self._ldap_call(self._l.set_option,option,invalue)
 
   def search_subschemasubentry_s(self,dn=''):
@@ -659,7 +700,7 @@ class ReconnectLDAPObject(SimpleLDAPObject):
   of all synchronous operation methods (search_s() etc.) are doing
   an automatic reconnect and rebind and will retry the very same
   operation.
-  
+
   This is very handy for broken LDAP server implementations
   (e.g. in Lotus Domino) which drop connections very often making
   it impossible to have a long-lasting control flow in the
@@ -804,6 +845,9 @@ class ReconnectLDAPObject(SimpleLDAPObject):
   def delete_ext_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.delete_ext_s,*args,**kwargs)
 
+  def extop_s(self,*args,**kwargs):
+    return self._apply_method_s(SimpleLDAPObject.extop_s,*args,**kwargs)
+
   def modify_ext_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.modify_ext_s,*args,**kwargs)
 
