@@ -1,5 +1,5 @@
 /* See http://www.python-ldap.org/ for details.
- * $Id: options.c,v 1.35 2009/11/13 08:45:50 stroeder Exp $ */
+ * $Id: options.c,v 1.38 2010/10/01 18:23:18 stroeder Exp $ */
 
 #include "common.h"
 #include "errors.h"
@@ -79,6 +79,9 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 #ifdef HAVE_TLS
     case LDAP_OPT_X_TLS:
     case LDAP_OPT_X_TLS_REQUIRE_CERT:
+#ifdef LDAP_OPT_X_TLS_CRLCHECK
+    case LDAP_OPT_X_TLS_CRLCHECK:
+#endif
 #ifdef LDAP_OPT_X_TLS_NEWCTX
     case LDAP_OPT_X_TLS_NEWCTX:
 #endif
@@ -90,6 +93,16 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
     case LDAP_OPT_X_SASL_SSF_MIN:
     case LDAP_OPT_X_SASL_SSF_MAX:
 #endif
+#ifdef LDAP_OPT_X_KEEPALIVE_IDLE
+    case LDAP_OPT_X_KEEPALIVE_IDLE:
+#endif
+#ifdef LDAP_OPT_X_KEEPALIVE_PROBES
+    case LDAP_OPT_X_KEEPALIVE_PROBES:
+#endif
+#ifdef LDAP_OPT_X_KEEPALIVE_INTERVAL
+    case LDAP_OPT_X_KEEPALIVE_INTERVAL:
+#endif
+
 	    /* integer value options */
 	    if (!PyArg_Parse(value, "i:set_option", &intval))
 		return 0;
@@ -97,6 +110,9 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 	    break;
     case LDAP_OPT_HOST_NAME:
     case LDAP_OPT_URI:
+#ifdef LDAP_OPT_DEFBASE
+    case LDAP_OPT_DEFBASE:
+#endif
     case LDAP_OPT_ERROR_STRING:
     case LDAP_OPT_MATCHED_DN:
 #ifdef HAVE_TLS
@@ -107,6 +123,9 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
     case LDAP_OPT_X_TLS_CIPHER_SUITE:
     case LDAP_OPT_X_TLS_RANDOM_FILE:
     case LDAP_OPT_X_TLS_DHFILE:
+#ifdef LDAP_OPT_X_TLS_CRLFILE
+    case LDAP_OPT_X_TLS_CRLFILE:
+#endif
 #endif
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SECPROPS:
@@ -220,6 +239,9 @@ LDAP_get_option(LDAPObject *self, int option)
 #ifdef HAVE_TLS
     case LDAP_OPT_X_TLS:
     case LDAP_OPT_X_TLS_REQUIRE_CERT:
+#ifdef LDAP_OPT_X_TLS_CRLCHECK
+    case LDAP_OPT_X_TLS_CRLCHECK:
+#endif
 #ifdef LDAP_OPT_X_TLS_PROTOCOL_MIN
     case LDAP_OPT_X_TLS_PROTOCOL_MIN:
 #endif
@@ -233,6 +255,15 @@ LDAP_get_option(LDAPObject *self, int option)
 #endif
 #ifdef LDAP_OPT_CONNECT_ASYNC
     case LDAP_OPT_CONNECT_ASYNC:
+#endif
+#ifdef LDAP_OPT_X_KEEPALIVE_IDLE
+    case LDAP_OPT_X_KEEPALIVE_IDLE:
+#endif
+#ifdef LDAP_OPT_X_KEEPALIVE_PROBES
+    case LDAP_OPT_X_KEEPALIVE_PROBES:
+#endif
+#ifdef LDAP_OPT_X_KEEPALIVE_INTERVAL
+    case LDAP_OPT_X_KEEPALIVE_INTERVAL:
 #endif
 	    /* Integer-valued options */
 	    if (self) LDAP_BEGIN_ALLOW_THREADS(self);
@@ -244,6 +275,9 @@ LDAP_get_option(LDAPObject *self, int option)
 
     case LDAP_OPT_HOST_NAME:
     case LDAP_OPT_URI:
+#ifdef LDAP_OPT_DEFBASE
+    case LDAP_OPT_DEFBASE:
+#endif
     case LDAP_OPT_ERROR_STRING:
     case LDAP_OPT_MATCHED_DN:
 #ifdef HAVE_TLS
@@ -254,6 +288,9 @@ LDAP_get_option(LDAPObject *self, int option)
     case LDAP_OPT_X_TLS_CIPHER_SUITE:
     case LDAP_OPT_X_TLS_RANDOM_FILE:
     case LDAP_OPT_X_TLS_DHFILE:
+#ifdef LDAP_OPT_X_TLS_CRLFILE
+    case LDAP_OPT_X_TLS_CRLFILE:
+#endif
 #endif
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SECPROPS:
